//Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
//This file is part of MagicCodes.
//
//MagicCodes is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//MagicCodes is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with Foobar; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.magiccodes.swingui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
//import javax.swing.event.*;
import java.util.Arrays;
import org.ninjasoft.magiccodes.logic.*;
import org.ninjasoft.magiccodes.plugins.*;
//import com.apple.eawt.*;

public class MainFrame extends JFrame {
    private boolean initialized = false;
    private Actions actions = new Actions();
    private JTextArea inputText = new JTextArea();
    private JEditorPane outputText = new JEditorPane();
    private JTextField keyText = new JTextField();
    private JTextField pluginDescriptionText = new JTextField();
    private DefaultComboBoxModel pluginComboModel = new DefaultComboBoxModel();
    private JComboBox inputCombo = new JComboBox(new String[] {"input is binary", "input is hex", "input is ASCII"});
    private JComboBox keyCombo = new JComboBox(new String[] {"key is binary", "key is hex", "key is ASCII"});
    private JComboBox outputCombo = new JComboBox(new String[] {"output to binary", "output to hex", "output to ASCII", "output to printable ASCII"});
    private JComboBox pluginCombo = new JComboBox(pluginComboModel);
    
    public void initialize() {
        initializeGui();
        initializeEvents();
        initializePlugins();
        //initializeApple();
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.setTitle("Magic Codes");
        this.setIconImage(new ImageIcon(MainFrame.class.getResource("/org/ninjasoft/magiccodes/icon64.gif")).getImage());
        inputCombo.setSelectedIndex(2);
        outputCombo.setSelectedIndex(3);
        keyCombo.setSelectedIndex(2);
    }
    
    /*
    public void initializeApple() {
        System.setProperty("com.apple.mrj.application.apple.menu.about.name", "Magic Codes");
        System.setProperty("com.apple.laf.useScreenMenuBar", "true");
        Application application = Application.getApplication();
        application.addApplicationListener(new ApplicationAdapter() {
            public void handleQuit(ApplicationEvent event) {
                dispose();
            }
        });
        application.setEnabledAboutMenu(false);
        application.setEnabledPreferencesMenu(false);
    }
    */
    
    private void initializePlugins() {
        Class pluginInterface = Plugin.class;
        PluginDiscoverer pd = new PluginDiscoverer();
        String[] plugins = pd.findMatchingPlugins(pluginInterface);
        Arrays.sort(plugins);
        for (int i=0; i<plugins.length; i++) {
            if (plugins[i].equals("org.ninjasoft.magiccodes.plugins.Plugin"))
                continue;
            Plugin item = null;
            try {
                item = (Plugin) Class.forName(plugins[i]).newInstance();
            } catch (Exception e) {
                System.out.println("Unable to load " + plugins[i] + ": " + e.toString());
            }
            if (item != null)
                this.pluginComboModel.addElement(new PluginItem(item));
        }
    }
    
    private void initializeGui() {
        if (initialized)
            return;
        initialized = true;
        this.setSize(500, 400);
        Dimension windowSize = this.getSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        this.setLocation(screenSize.width/2 - windowSize.width/2, screenSize.height/2 - windowSize.height/2);
        Container content = this.getContentPane();
        content.setLayout(new BorderLayout());
        JPanel pane = new JPanel();
        pane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        content.add(pane, BorderLayout.CENTER);
        pane.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        // ROW 2 //////////////////////////////////////
        c.gridx = c.gridy = 0;
        c.gridwidth = c.gridheight = 1;
        c.weightx = c.weighty = 0;
        c.anchor = GridBagConstraints.WEST;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(3, 3, 3, 3);
        pane.add(new JLabel("Input:"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        pane.add(inputCombo, c);
        // ROW 2 //////////////////////////////////////
        c.gridx = 0;
        c.gridy++;
        c.gridwidth = 3;
        c.weightx = 3;
        c.weighty = 2;
        c.fill = GridBagConstraints.BOTH;
        pane.add(new JScrollPane(inputText), c);
        // ROW 3 //////////////////////////////////////
        c.gridx = 0;
        c.gridy++;
        c.gridwidth = 1;
        c.weightx = c.weighty = 0;
        c.fill = GridBagConstraints.NONE;
        pane.add(new JLabel("Key:"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        pane.add(keyText, c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        pane.add(keyCombo, c);
        // ROW 4 //////////////////////////////////////
        c.gridx = 0;
        c.gridy++;
        c.gridwidth = 1;
        c.weightx = c.weighty = 0;
        c.fill = GridBagConstraints.NONE;
        pane.add(new JLabel("Plugin:"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        pane.add(pluginCombo, c);
        // ROW 5 //////////////////////////////////////
        c.gridx = 0;
        c.gridy++;
        c.gridwidth = 3;
        c.weightx = c.weighty = 0;
        c.fill = GridBagConstraints.HORIZONTAL;
        pluginDescriptionText.setBorder(BorderFactory.createEmptyBorder());
        pluginDescriptionText.setBackground(pane.getBackground());
        pluginDescriptionText.setOpaque(false);
        pane.add(pluginDescriptionText, c);
        // ROW 6 //////////////////////////////////////
        c.gridx = 0;
        c.gridy++;
        c.gridwidth = 1;
        c.weightx = c.weighty = 0;
        c.fill = GridBagConstraints.NONE;
        pane.add(new JLabel("Output:"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        pane.add(outputCombo, c);
        c.gridx++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        JButton button = new JButton("Calculate");
        button.addActionListener(actions);
        button.setActionCommand("go");
        pane.add(button, c);
        // ROW 7 //////////////////////////////////////
        c.gridx = 0;
        c.gridy++;
        c.gridwidth = 3;
        c.weightx = 3;
        c.weighty = 2;
        c.anchor = GridBagConstraints.WEST;
        c.fill = GridBagConstraints.BOTH;
        pane.add(new JScrollPane(outputText), c);
        
        outputText.setEditable(false);
    }
    
    private void initializeEvents() {
        pluginCombo.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                updateKeyFieldState();
                updatePluginDescription();
            }
        });
    }
    
    private void updateKeyFieldState() {
        boolean enabled = ((PluginItem) this.pluginComboModel.getSelectedItem()).getPlugin().usesKey();
        keyText.setEnabled(enabled);
        keyCombo.setEnabled(enabled);
    }
    
    private void updatePluginDescription() {
        String description = ((PluginItem) this.pluginComboModel.getSelectedItem()).getPlugin().getDescription();
        pluginDescriptionText.setText(description);
    }
    
    private void doIt() {
        Processor processor = new Processor();
        Plugin plugin = ((PluginItem) this.pluginComboModel.getSelectedItem()).getPlugin();
        if (plugin.isInformational())
            this.outputCombo.setSelectedIndex(3);
        processor.addPlugin(plugin);
        processor.setInputType(this.inputCombo.getSelectedIndex() + 1);
        processor.setOutputType(this.outputCombo.getSelectedIndex() + 1);
        processor.setKeyType(this.keyCombo.getSelectedIndex() + 1);
        try{
            System.out.println("Calculating...");
            String result = processor.doAction(inputText.getText(), keyText.getText());
            outputText.setText("");
            if ((this.outputCombo.getSelectedIndex() + 1) == 4)
                outputText.setContentType("text/html");
            else
                outputText.setContentType("text/plain");
            outputText.setText(result);
            System.out.println("Done");
        }catch(FormatException e) {
            System.out.println("Error");
            outputText.setText(">>> MagicCodes found an error with your input format:\n\n" + e.toString());
            outputText.setContentType("text/plain");
        }catch(Throwable t) {
            t.printStackTrace();
            outputText.setText(">>> Problem running plugin:\n\n" + t.toString());
            outputText.setContentType("text/plain");
        }
        outputText.setCaretPosition(0);
    }
    
    public class Actions implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            String command = e.getActionCommand();
            command = command == null ? "" : command;
            if (command.equals("go"))
                doIt();
        }
    }

    public void dispose() {
        System.exit(0);
    }
    
    public void setVisible(boolean b) {
        initialize();
        super.setVisible(b);
    }

    public static void main(String[] args) {
        new MainFrame().setVisible(true);
    }
}
